/*
 * ====================================================================
 * Copyright (c) 2008 JavaGit Project.  All rights reserved.
 *
 * This software is licensed using the GNU LGPL v2.1 license.  A copy
 * of the license is included with the distribution of this source
 * code in the LICENSE.txt file.  The text of the license can also
 * be obtained at:
 *
 *   http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 *
 * For more information on the JavaGit project, see:
 *
 *   http://www.javagit.com
 * ====================================================================
 */

package edu.nyu.cs.javagit.api.commands;

import java.io.File;
import java.io.IOException;
import java.util.List;

import edu.nyu.cs.javagit.api.JavaGitException;
import edu.nyu.cs.javagit.client.ClientManager;
import edu.nyu.cs.javagit.client.IClient;
import edu.nyu.cs.javagit.client.IGitAdd;
import edu.nyu.cs.javagit.utilities.CheckUtilities;

/**
 * <code>GitAdd</code> provides an interface for adding files or directory tree into git
 * repository
 */
public final class GitAdd {
  /**
   * This command adds the current content of new or modified files to the index, thus staging that
   * content for inclusion in the next commit.
   * 
   * @param repositoryPath
   *          The path to the repository to commit against. A non-zero length argument is required
   *          for this parameter, otherwise a <code>NullPointerException</code> or
   *          <code>IllegalArgumentException</code> will be thrown.
   * @param options
   *          GitAddOptions that are passed to the &lt;git-add&gt; command.
   * @param paths
   *          List of file paths that will be added to the index.
   * @return GitAddResponse Response generated by the add command.
   * @exception IOException
   *              There are many reasons for which an <code>IOException</code> may be thrown if -
   *              <ul>
   *              <li>repositoryPath is not valid</li>
   *              <li>filenames assigned do not exist</li>
   *              <li>files or repository path does not have access rights</li>
   *              </ul>
   * @exception JavaGitException
   *              Exception thrown
   *              <ul>
   *              <li>if the repositoryPath is null.</li>
   *              <li>if the &lt;git-add&gt; command generates error.</li>
   *              </ul>
   */
  public GitAddResponse add(File repositoryPath, GitAddOptions options, List<File> paths)
      throws IOException, JavaGitException {
    CheckUtilities.checkFileValidity(repositoryPath.getAbsoluteFile());
    IClient client = ClientManager.getInstance().getPreferredClient();
    IGitAdd gitAdd = client.getGitAddInstance();
    return gitAdd.add(repositoryPath, options, paths);
  }

  /**
   * This command adds the current content of new or modified files to the index, with verbose
   * option set.
   * 
   * @param repositoryPath
   *          The path to the repository to commit against. A non-zero length argument is required
   *          for this parameter, otherwise a <code>NullPointerException</code> or
   *          <code>IllegalArgumentException</code> will be thrown.
   * @param paths
   *          List of file paths that will be added to the index.
   * @return GitAddResponse Response generated by the add command.
   * @exception IOException
   *              There are many reasons for which an <code>IOException</code> may be thrown if -
   *              <ul>
   *              <li>repositoryPath is not valid</li>
   *              <li>filenames assigned do not exist</li>
   *              <li>files or repository path does not have access rights</li>
   *              </ul>
   * @exception JavaGitException
   *              Exception thrown
   *              <ul>
   *              <li>if the repositoryPath is null.</li>
   *              <li>if the &lt;git-add&gt; command generates error.</li>
   *              </ul>
   */
  public GitAddResponse addVerbose(File repositoryPath, List<File> paths) throws IOException,
      JavaGitException {
    IClient client = ClientManager.getInstance().getPreferredClient();
    IGitAdd gitAdd = client.getGitAddInstance();
    return gitAdd.addVerbose(repositoryPath, paths);
  }

  /**
   * This command adds the current content of new or modified files to the index, with Force option
   * set i.e. it adds the ignored files as well.
   * 
   * @param repositoryPath
   *          The path to the repository to commit against. A non-zero length argument is required
   *          for this parameter, otherwise a <code>NullPointerException</code> or
   *          <code>IllegalArgumentException</code> will be thrown.
   * @param paths
   *          <code>List</code> of file paths that will be added to the index.
   * @return GitAddResponse Response generated by the add command.
   * @exception IOException
   *              There are many reasons for which an <code>IOException</code> may be thrown if -
   *              <ul>
   *              <li>repositoryPath is not valid</li>
   *              <li>filenames assigned do not exist</li>
   *              <li>files or repository path does not have access rights</li>
   *              </ul
   * @exception JavaGitException
   *              Exception thrown
   *              <ul>
   *              <li>if the repositoryPath is null.</li>
   *              <li>if the &lt;git-add&gt; command generates error.</li>
   *              </ul>
   */
  public GitAddResponse addWithForce(File repositoryPath, List<File> paths) throws IOException,
      JavaGitException {
    IClient client = ClientManager.getInstance().getPreferredClient();
    IGitAdd gitAdd = client.getGitAddInstance();
    return gitAdd.addWithForce(repositoryPath, paths);
  }

  /**
   * This command just executes &lt;git-add&gt; in dry run mode.
   * 
   * @param repositoryPath
   *          The path to the repository to commit against. A non-zero length argument is required
   *          for this parameter, otherwise a <code>NullPointerException</code> or
   *          <code>IllegalArgumentException</code> will be thrown.
   * @param paths
   *          <code>List</code> of file paths that will be added to the index.
   * @return results <code>GitAddResponse</code> from the add command.
   * @exception IOException
   *              There are many reasons for which an <code>IOException</code> could be thrown if -
   *              <ul>
   *              <li>repositoryPath is not valid</li>
   *              <li>filenames assigned do not exist</li>
   *              <li>files or repository path does not have access rights</li>
   *              </ul>
   * @exception JavaGitException
   *              Exception thrown
   *              <ul>
   *              <li>if the repositoryPath is null.</li>
   *              <li>if the &lt;git-add&gt; command generates error.</li>
   *              </ul>
   */
  public GitAddResponse addDryRun(File repositoryPath, List<File> paths) throws IOException,
      JavaGitException {
    IClient client = ClientManager.getInstance().getPreferredClient();
    IGitAdd gitAdd = client.getGitAddInstance();
    return gitAdd.addDryRun(repositoryPath, paths);
  }

  /**
   * This command adds the current content of new or modified file to the index, thus staging that
   * content for inclusion in the next commit.
   * 
   * @param repositoryPath
   *          The path to the repository to commit against. A non-zero length argument is required
   *          for this parameter, otherwise a <code>NullPointerException</code> or
   *          <code>IllegalArgumentException</code> will be thrown.
   * @param options
   *          <code>GitAddOptions</code> that are passed to the &lt;git-add&gt; command.
   * @param file File to be added to the index.
   * @return GitAddResponse object.
   * @exception IOException There are many reasons for which an <code>IOException</code>
   *              may be thrown if -
   *              <ul>
   *              <li>repositoryPath is not valid</li>
   *              <li>filenames assigned do not exist</li>
   *              <li>files or repository path does not have access rights</li>
   *              </ul>
   * @exception JavaGitException
   *              Exception thrown
   *              <ul>
   *              <li>if the repositoryPath is null.</li>
   *              <li>if the &lt;git-add&gt; command generates error.</li>
   *              </ul>
   */
  public GitAddResponse add(File repositoryPath, GitAddOptions options, File file)
      throws IOException, JavaGitException {
    IClient client = ClientManager.getInstance().getPreferredClient();
    IGitAdd gitAdd = client.getGitAddInstance();
    return gitAdd.add(repositoryPath, options, file);
  }

  /**
   * Adds list of files to the index.
   * 
   * @param repositoryPath
   *          File path pointing to the root of the repository.
   * @param file
   *          <code>File</code> will added to the index.
   * @return GitAddResponse object.
   * @throws JavaGitException Thrown when there is an error while running the add command.
   * @throws IOException There are many reasons for which an <code>IOException</code>
   *           may be thrown. Examples include:
   *           <ul>
   *           <li>a directory doesn't exist</li>
   *           <li>access to a file is denied</li>
   *           <li>a command is not found on the PATH</li>
   *           </ul>
   */
  public GitAddResponse add(File repositoryPath, File file) throws IOException, JavaGitException {
    IClient client = ClientManager.getInstance().getPreferredClient();
    IGitAdd gitAdd = client.getGitAddInstance();
    return gitAdd.add(repositoryPath, file);
  }

  /**
   * Adds list of files to the index with no options set.
   * 
   * @param repositoryPath
   *          File path pointing to the root of the repository
   * @param paths
   *          <code>List</code> of file paths that are going to be added to the index.
   * @return GitAddResponse object.
   * @throws JavaGitException Thrown when there is an error while running the add command.
   * @throws IOException There are many reasons for which an <code>IOException</code>
   *           may be thrown. Examples include:
   *           <ul>
   *           <li>a directory doesn't exist</li>
   *           <li>access to a file is denied</li>
   *           <li>a command is not found on the PATH</li>
   *           </ul>
   */
  public GitAddResponse add(File repositoryPath, List<File> paths) throws JavaGitException,
      IOException {
    IClient client = ClientManager.getInstance().getPreferredClient();
    IGitAdd gitAdd = client.getGitAddInstance();
    return gitAdd.add(repositoryPath, paths);
  }

}
